#include "project.h"

#include <stdbool.h>
#include <stdio.h>
#include <string.h>

#include "spi_dma.h"

#define ARRAY_SIZE(x) (sizeof(x) / sizeof(x[0]))

enum { MAX_ARRAY_SIZE = 32 };
enum { FEED_FORM_CMD = 0xC };

void print_array(const uint16_t *data, const size_t size);

int main(void)
{
    const uint16_t data_send[MAX_ARRAY_SIZE] = {
        0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
        0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
        0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
        0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08
    };

    uint16_t data_get[MAX_ARRAY_SIZE] = {0};

    CyGlobalIntEnable;

    UART_Start();
    SPI_Start();

    UART_PutChar(FEED_FORM_CMD); // clear the screen
    UART_PutString("\t\tTest transfer memory->SPI with DMA\r\n");
    
    config_spi_dma();

    // transfer with configurable DMA
    UART_PutString("\r\nTransfering via configurable DMA...\r\n");
    send_spi_dma(data_send, data_get, ARRAY_SIZE(data_get));

    // printing results
    UART_PutString("Re configurable DMA transfer done!\r\n");
    UART_PutString("Printing the source array\r\n");
    print_array(data_send, ARRAY_SIZE(data_send));
    UART_PutCRLF();
    UART_PutString("Printing the destination array\r\n");
    print_array(data_get, ARRAY_SIZE(data_get));
    UART_PutCRLF();
    
    // setting array to zero
    UART_PutString("Setting the destination array to zero\r\n");
    memset(data_get, 0, ARRAY_SIZE(data_get));
    UART_PutString("Printing the destination array\r\n");
    print_array(data_get, ARRAY_SIZE(data_get));
    UART_PutCRLF();
    
    UART_PutString("Transfering just 5 bytes via configurable DMA...\r\n");
    send_spi_dma(data_send, data_get, 5);

    // printing results
    UART_PutString("Re configurable DMA transfer done!\r\n");
    UART_PutString("Printing the source array\r\n");
    print_array(data_send, 5);
    UART_PutCRLF();
    UART_PutString("Printing the destination array\r\n");
    print_array(data_get, 5);

    while (1) {
        #if 0
        memset(data_get, 0, MAX_ARRAY_SIZE);
        send_spi_dma(data_send, data_get, MAX_ARRAY_SIZE);
        CyDelay(10);
        memset(data_get, 0, MAX_ARRAY_SIZE);
        send_spi_dma(data_send, data_get, 5);
        CyDelay(10);
        memset(data_get, 0, MAX_ARRAY_SIZE);
        send_spi_dma(data_send, data_get, 10);
        CyDelay(10);
        #endif
    }
}

void print_array(const uint16_t *data, const size_t size)
{
    for (size_t i = 0; i < size; i++) {
        UART_PutHexInt(data[i]);
        
        if (0 != (i % 8)) {
            UART_PutString(", ");
        } else {
            UART_PutString("\r\n");
        }
    }
}

/* [] END OF FILE */
