#include "project.h"
#include "spi_dma.h"

static const uint8_t DMA_REQUEST_PER_BURST = 1;
static const uint8_t DMA_BYTES_PER_BURST = 2;
static uint8_t DMA_Rx_Chan = 0;
static uint8_t DMA_Tx_Chan = 0;
static uint8_t DMA_Rx_TD = 0;
static uint8_t DMA_Tx_TD = 0;

void config_spi_dma(void)
{
    // Allocate DMA Channels
    DMA_Rx_Chan = DMA_Rx_DmaInitialize(DMA_BYTES_PER_BURST, DMA_REQUEST_PER_BURST,
                                        HI16(CYDEV_PERIPH_BASE), HI16(CYDEV_SRAM_BASE));
    
    DMA_Tx_Chan = DMA_Tx_DmaInitialize(DMA_BYTES_PER_BURST, DMA_REQUEST_PER_BURST,
                                        HI16(CYDEV_SRAM_BASE), HI16(CYDEV_PERIPH_BASE));
    
    // Allocate TDs
    DMA_Rx_TD = CyDmaTdAllocate();
    DMA_Tx_TD = CyDmaTdAllocate();
    
    CyDmaChSetInitialTd(DMA_Rx_Chan, DMA_Rx_TD);
    CyDmaChSetInitialTd(DMA_Tx_Chan, DMA_Tx_TD);
}

void send_spi_dma(const uint16_t *tx_data, uint16_t *rx_data, const size_t size)
{    
    // disable the DMAs before reconfiguring them
    //CyDmaChDisable(DMA_Rx_Chan);
    //CyDmaChDisable(DMA_Tx_Chan);

    // size =  size * 2 because it's a 16bit transfer
    CyDmaTdSetConfiguration(DMA_Rx_TD, size * 2, CY_DMA_DISABLE_TD, CY_DMA_TD_INC_DST_ADR);
    CyDmaTdSetConfiguration(DMA_Tx_TD, size * 2, CY_DMA_DISABLE_TD, CY_DMA_TD_INC_SRC_ADR);
    
    CyDmaTdSetAddress(DMA_Rx_TD, LO16((uint32)SPI_BSPIM_sR16_Dp_u0__16BIT_F1_REG), LO16((uint32)rx_data));
    // SPI_TXDATA_PTR = SPI_BSPIM_sR16_Dp_u0__F0_REG
    CyDmaTdSetAddress(DMA_Tx_TD, LO16((uint32)tx_data), LO16((uint32)SPI_BSPIM_sR16_Dp_u0__16BIT_F0_REG));

    SS_Write(0);
    
    // enable the DMAs
    CyDmaChEnable(DMA_Rx_Chan, 1);
    CyDmaChEnable(DMA_Tx_Chan, 1);

    // Waiting for the /SS to go high
    UART_PutString("Waiting for the SPI to be done\r\n");
    while(!(SPI_ReadTxStatus() & SPI_STS_SPI_DONE)) {
    }
    SS_Write(1);
    LED_Write(1);
}

/* [] END OF FILE */
